@@ -30,6 +30,13 @@
 
 /* 
  * $Log: callbacks.c,v $
+ *
+ * Revision 2.4.1 2007/08/02 tissieres
+ * bugfix extract hash
+ *
+ * Revision 2.4 2007/06/21 oechslin tissieres
+ * bugfix, ISO charset for nthash in fill_tree, file insensitive for *nix
+ *
  * Revision 2.3.4 2007/02/19 tissieres
  * bkhive-1.1.0, show_dialog
  *
@@ -61,6 +68,8 @@
 #include <gtk/gtk.h>
 #ifndef WIN32
 #include <sys/times.h>
+#include <dirent.h>
+#include <string.h>
 #else /* WIN32 */
 #include <sys/time.h>
 #include <windows.h>
@@ -328,11 +337,11 @@ int change_tree(int hash) {
   gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL(list_store), &iter, line);
 
   if (info[hash][0])
-    gtk_list_store_set(list_store, &iter, 0, userid[hash], 1, g_convert(info[hash], -1, "UTF-8", "ISO_8859-1", NULL, &j, NULL), 2, g_convert(password1[hash], -1, "UTF-8", "CP437", NULL, &j, NULL), 3, g_convert(password2[hash], -1, "UTF-8", "CP437", NULL, &j, NULL), 4, g_convert(password[hash], -1, "UTF-8", "CP437", NULL, &j, NULL), -1);
+    gtk_list_store_set(list_store, &iter, 0, userid[hash], 1, g_convert(info[hash], -1, "UTF-8", "ISO_8859-1", NULL, &j, NULL), 2, g_convert(password1[hash], -1, "UTF-8", "CP437", NULL, &j, NULL), 3, g_convert(password2[hash], -1, "UTF-8", "CP437", NULL, &j, NULL), 4, g_convert(password[hash], -1, "UTF-8", "ISO_8859-1", NULL, &j, NULL), -1);
   else {
     for (j=0; j<8; j++) snprintf(temp+j*2,3,  "%02hhx",lm_hash1[hash][j]);
     for (j=0; j<8; j++) snprintf(temp+(8+j)*2,3, "%02hhx",lm_hash2[hash][j]);
-    gtk_list_store_set(list_store, &iter, 0, hash, 1, temp, 2, g_convert(password1[hash], -1, "UTF-8", "CP437", NULL, &j, NULL), 3, g_convert(password2[hash], -1, "UTF-8", "CP437", NULL, &j, NULL), 4, g_convert(password[hash], -1, "UTF-8", "CP437", NULL, &j, NULL), -1);
+    gtk_list_store_set(list_store, &iter, 0, hash, 1, temp, 2, g_convert(password1[hash], -1, "UTF-8", "CP437", NULL, &j, NULL), 3, g_convert(password2[hash], -1, "UTF-8", "CP437", NULL, &j, NULL), 4, g_convert(password[hash], -1, "UTF-8", "ISO_8859-1", NULL, &j, NULL), -1);
   }
   return 1;
 }
@@ -359,11 +368,11 @@ int fill_tree(void) {
     gtk_list_store_append(list_store, &iter);
 
     if (info[i][0])
-      gtk_list_store_set(list_store, &iter, 0, userid[i], 1, g_convert(info[i], -1, "UTF-8", "ISO_8859-1", NULL, &j, NULL), 2, g_convert(password1[i], -1, "UTF-8", "CP437", NULL, &j, NULL), 3, g_convert(password2[i], -1, "UTF-8", "CP437", NULL, &j, NULL), 4, g_convert(password[i], -1, "UTF-8", "CP437", NULL, &j, NULL), -1);
+      gtk_list_store_set(list_store, &iter, 0, userid[i], 1, g_convert(info[i], -1, "UTF-8", "ISO_8859-1", NULL, &j, NULL), 2, g_convert(password1[i], -1, "UTF-8", "CP437", NULL, &j, NULL), 3, g_convert(password2[i], -1, "UTF-8", "CP437", NULL, &j, NULL), 4, g_convert(password[i], -1, "UTF-8", "ISO_8859-1", NULL, &j, NULL), -1);
     else {
       for (j=0; j<8; j++) snprintf(temp+j*2,3,  "%02hhx",lm_hash1[i][j]);
       for (j=0; j<8; j++) snprintf(temp+(8+j)*2,3, "%02hhx",lm_hash2[i][j]);
-      gtk_list_store_set(list_store, &iter, 0, i, 1, temp, 2, g_convert(password1[i], -1, "UTF-8", "CP437", NULL, &j, NULL), 3, g_convert(password2[i], -1, "UTF-8", "CP437", NULL, &j, NULL), 4, g_convert(password[i], -1, "UTF-8", "CP437", NULL, &j, NULL), -1);
+      gtk_list_store_set(list_store, &iter, 0, i, 1, temp, 2, g_convert(password1[i], -1, "UTF-8", "CP437", NULL, &j, NULL), 3, g_convert(password2[i], -1, "UTF-8", "CP437", NULL, &j, NULL), 4, g_convert(password[i], -1, "UTF-8", "ISO_8859-1", NULL, &j, NULL), -1);
     }
   }
   return 1;
@@ -452,7 +461,7 @@ void write_table_name_from_type(int type, char *buf) {
   
 }
  
-void update_statusbar(int k) {
+void update_statusbar(int msg, int k) {
   extern clock_t n_start_time, laps_time; 
   extern struct tms n_times;  
   extern int first_table, last_table, cols, n_found, hashes, ticks;
@@ -473,8 +482,14 @@ void update_statusbar(int k) {
   laps_time = clock();
 #endif /* WIN32 */
   
-  buff = g_strdup_printf (" Table set: %s | Tables in use: %d to %d :%2d%% | Passwords:%d/%d | Time elapsed: %.2f", buff2, first_table+1, last_table+1, 100-(100*k)/cols, n_found, hashes, (float)(laps_time - n_start_time)/ticks);
-  
+  switch(msg) {
+  case MSG_RESOLV_NT_HASH:
+    buff = g_strdup_printf (" Resolving NT password... (%d combinations)| Passwords:%d/%d | Time elapsed: %.2f", k, n_found, hashes, (float)(laps_time - n_start_time)/ticks);
+    break;
+  case MSG_DEFAULT:
+    buff = g_strdup_printf (" Table set: %s | Tables in use: %d to %d :%2d%% | Passwords:%d/%d | Time elapsed: %.2f", buff2, first_table+1, last_table+1, 100-(100*k)/cols, n_found, hashes, (float)(laps_time - n_start_time)/ticks);
+  }
+
   gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, buff);
   g_free (buff);
 }
@@ -501,35 +516,65 @@ on_dump1_clicked                       (GtkToolButton   *toolbutton,
 }
 
 
+
 void
 on_ok_button2_clicked                  (GtkButton       *button,
                                         gpointer         user_data)
 {
 
   char *filename;
+ 
+  filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fc));
+
+  if (extract_hashes_from_dir(filename)) {
+    gtk_widget_destroy(fc);
+  } 
+}
+
+int
+extract_hashes_from_dir(char *dir)
+{
+
   char command[500];
 #ifndef WIN32
   char tmp_file[] = "/tmp/ophcrack.tmp";
   char tmp_file2[] = "/tmp/ophcrack2.tmp";
+  char system_file[] = "system";
+  char sam_file[] = "sam";
+  DIR *dp;
+  struct dirent *ep;
 #else
   char temp_path[450];
   char tmp_file[500]; 
   char tmp_file2[500];
 #endif  
   GtkWidget *tmp;
-  
-  filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fc));
+   
 #ifndef WIN32
-  sprintf(command, "bkhive \"%s/SYSTEM\" %s 2>&1 1>/dev/null", filename, tmp_file2);
+  /*make SYSTEM and SAM case-insensitive*/
+  dp = opendir (dir);
+  if (dp != NULL) {
+    while (ep = readdir(dp)) {
+      if (strlen(ep->d_name) == 6)
+	if (!strncasecmp(ep->d_name, system_file, 6))
+	  strncpy(system_file, ep->d_name, 6);
+      if (strlen(ep->d_name) == 3)
+	if (!strncasecmp(ep->d_name, sam_file, 3))
+	  strncpy(sam_file, ep->d_name, 3);
+    }
+    closedir(dp);
+  }  
+  
+  sprintf(command, "bkhive \"%s/%s\" %s 2>&1 1>/dev/null", dir, system_file, tmp_file2);
   system(command);
-  sprintf(command, "samdump2 \"%s/SAM\" %s > %s 2>/dev/null",  filename, tmp_file2, tmp_file);
+  sprintf(command, "samdump2 \"%s/%s\" %s > %s 2>/dev/null",  dir, sam_file, tmp_file2, tmp_file);
 #else /* WIN32 */
   GetTempPath(449, temp_path);
   sprintf(tmp_file, "%sophcrack.tmp", temp_path);
   sprintf(tmp_file2, "%sophcrack2.tmp", temp_path);
-  sprintf(command, "..\\win32_tools\\bkhive \"%s\\SYSTEM\" \"%s\"", filename, tmp_file2);
+  sprintf(command, "..\\win32_tools\\bkhive \"%s\\SYSTEM\" \"%s\"", dir, tmp_file2);
   system(command);
-  sprintf(command, "..\\win32_tools\\samdump2 \"%s\\SAM\" \"%s\" > \"%s\"",  filename, tmp_file2, tmp_file);
+  sprintf(command, "..\\win32_tools\\samdump2 \"%s\\SAM\" \"%s\" > \"%s\"",  dir, tmp_file2, tmp_file);
 #endif
 
   system(command);
@@ -537,25 +582,23 @@ on_ok_button2_clicked                  (GtkButton       *button,
     fill_tree();
     unlink(tmp_file);
     unlink(tmp_file2);
-
-    gtk_widget_destroy(fc);
-
     gtk_widget_set_sensitive(main_window, 1);
     tmp = lookup_widget(main_window, "launch1");
     gtk_widget_set_sensitive(tmp, 1);
     tmp = lookup_widget(main_window, "save_button");
     gtk_widget_set_sensitive(tmp, 1);
-
+    return 1;
   }
   else {
     unlink(tmp_file);
     GtkWidget* error_dialog = create_dialog1();
     gtk_widget_show(error_dialog);
+    return 0;
   }
+  return 0;
 }
 
 
-
 void
 on_save_ok_button_clicked              (GtkButton   *button,
                                         gpointer         user_data)
@@ -572,6 +615,7 @@ on_save_ok_button_clicked              (GtkButton   *button,
   char *filename;
   FILE *save_file;
   unsigned char *tmp;
+  unsigned char empty_bytes[16]={0};
 
   filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fc2));
   if (access(filename, F_OK) == 0)
@@ -590,7 +634,7 @@ on_save_ok_button_clicked              (GtkButton   *button,
     for (i=0; i<8; i++) fprintf(save_file,"%02hhx",lm_hash1[h][i]);
     for (i=0; i<8; i++) fprintf(save_file,"%02hhx",lm_hash2[h][i]);
     fprintf(save_file,":");
-    if (strcmp(nt_hash[h],""))
+    if (memcmp(nt_hash[h],empty_bytes,16))
       for (i=0; i<16; i++) fprintf(save_file,"%02hhx",nt_hash[h][i]);
     fprintf(save_file,":");
     /* thanks to BaLP */
@@ -1177,18 +1221,18 @@ hide_readahead_window(void) {
 void activate_launch_toggle (void) {
 
   GtkWidget *tmp;
-  char tmp_file[] = "/tmp/ophcrack.tmp";
+/*   char tmp_file[] = "/tmp/ophcrack.tmp"; */
 
-  if (read_file(tmp_file)) {
-    fill_tree();
+/*   if (read_file(tmp_file)) { */
+/*     fill_tree(); */
     
-    gtk_widget_set_sensitive(main_window, 1);
-    tmp = lookup_widget(main_window, "launch1");
-    gtk_widget_set_sensitive(tmp, 1);
-    tmp = lookup_widget(main_window, "save_button");
-    gtk_widget_set_sensitive(tmp, 1);
+/*     gtk_widget_set_sensitive(main_window, 1); */
+/*     tmp = lookup_widget(main_window, "launch1"); */
+/*     gtk_widget_set_sensitive(tmp, 1); */
+/*     tmp = lookup_widget(main_window, "save_button"); */
+/*     gtk_widget_set_sensitive(tmp, 1); */
     
-  }
+/*   } */
   tmp = lookup_widget(main_window, "launch1");
   gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(tmp), TRUE);
 
