@@ -30,6 +30,12 @@
 
 /* 
  * $Log: ophcrack.c,v $
+ * Revision 2.4.1 2007/08/02 tissieres
+ * bugfix extract hash
+ *
+ * Revision 2.4  2007/06/22 oechslin
+ * Added support for accented characters that map to ASCII
+ *
  * Revision 2.3.4 2007/02/19 tissieres
  * German chars support, Bug fixes in hash counters, 
  * LiveCD auto-detect tables, minor changes
@@ -136,8 +142,9 @@ unsigned char
 
 int to_go, hashes=0, n=0, empty_lm=0, done1[MAX_HASH+1]={0},done2[MAX_HASH+1]={0}, userid[MAX_HASH+1], done_nt[MAX_HASH+1]={0};
 
-char directory[128], info[MAX_HASH+1][64]={{0}}, password[MAX_HASH+1][16]={{0}}, 
+char directory[128], info[MAX_HASH+1][64]={{0}},  
     password1[MAX_HASH+1][8]={{0}}, password2[MAX_HASH+1][8]={{0}}; 
+unsigned char password[MAX_HASH+1][16]={{0}};
 
 #ifndef WIN32
 FILE *startfile[20],*endfile[20],*indexfile[20];
@@ -967,59 +974,127 @@ int remove_line(int hash) {
   hashes--;
 }
 
-int resolve_nt_hash(char p1[], char p2[], char h[], char pw[])
+int resolve_nt_hash(char p1[], char p2[], char h[], unsigned char pw[], int insist)
 {
-  unsigned int i,x;
-  unsigned int k=0,l;
-  unsigned char md4[17],*p;
+  unsigned int i,j, combinations;
+  unsigned int l,max;
+  unsigned char md4[17],*p, orig[15]={0};
+  
+  static unsigned char is_multi[256]={
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
+    1, 8, 2, 2, 2, 8, 2, 2, 2, 8, 2, 2, 2, 2, 2, 8, 
+    2, 2, 2, 2, 2, 8, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
 
-  bzero(pw,15);
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
+    1, 1, 1, 1, 2, 1, 1, 2, 1, 2, 1, 1, 1, 1, 1, 1, 
+    1, 2, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 2, 1, 1, 1, 
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1};
+
+  static unsigned char multi_char[256][12]= {
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    {0}, {'A','a',0xe0,0xe2,0xc0,0xc2,0xe3,0xc3}, "Bb", "Cc", "Dd", 
+    {'E','e',0xc8,0xe8,0xca,0xea,0xeb,0xcb}, "Ff", "Gg", "Hh", {'I','i',0xee,0xef,0xce,0xcf,0xed,0xcd},
+    "Jj", "Kk", "Ll", "Mm", "Nn", {'O','o',0xf4,0xd4,0xf3,0xf5,0xd3,0xd5},
+    "Pp", "Qq", "Rr", "Ss", "Tt", {'U','u',0xf9,0xfb,0xd9,0xdb,0xfa,0xda}, "Vv", 
+    "Ww", "Xx", "Yy", "Zz", {0}, {0}, {0}, {0}, {0},
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    {0}, {0}, {0}, {0}, {0xc4,0xe4}, {0}, {0}, {0xe7,0xc7}, {0}, {0xc9,0xe9}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    {0}, {0xd1,0xf1}, {0}, {0}, {0}, {0}, {0xd6,0xf6}, {0}, {0}, {0}, {0}, {0}, {0xdc,0xfc}, {0}, {0}, {0xdf}, 
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, 
+    {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}} ;
+
+  
+  static int max_multi=8;
+  
+  bzero(pw,16);
   strcpy(pw,p1);
   strcpy(&pw[7],p2);
   
-  md4[16]=0;
+#define MAX(a,b) ((a)>(b)?(a):(b))
+#define MIN(a,b) ((a)<(b)?(a):(b))
 
   //clean special chars
 
+  l=strlen(pw);
+
   p = (unsigned char *)pw;
-  for (i=0; i<strlen(p); i++)
-    if (p[i]>=128)
+  for (i=0; i<l; i++)
+    if (p[i]>=128) 
       switch(p[i]){
-      case 0x8e: p[i]=0xc4;break;
-      case 0x99: p[i]=0xd6;break;
-      case 0x9a: p[i]=0xdc;break;
-      case 0xe1: p[i]=0xdf;break;
+      case 0x8e: p[i]=0xc4;break; // capital a umlaut
+      case 0x99: p[i]=0xd6;break; // capital o umlaut
+      case 0x9a: p[i]=0xdc;break; // capital u umlaut  
+      case 0xe1: p[i]=0xdf;break; // double s
+      case 0xa5: p[i]=0xd1;break; // capital n tilda
+      case 0x90: p[i]=0xc9;break; // capital e accute
+      case 0x80: p[i]=0xc7;break; // capital c cedilla
       }
-  
-  for (i=0; i<16384; i++) {
-    l = k ^ i ^ (i>>1);
-    k = i ^ (i>>1);
-    p = (unsigned char *)pw; while ((l=l>>1)) p++;
-    if (*p<128)
-      *p=(*p==(x=tolower(*p))?toupper(*p):x);
-    else 
-      *p=(*p>0xe0?*p-0x20:*p+0x20);
-    
-    make_nthash(pw,(char *)md4);
-    
-    if (!memcmp(md4,h,16)) {
-      p = (unsigned char *)pw;
-      for (i=0; i<strlen(p); i++)
-	if (p[i]>=128)
-	  switch(p[i]){
-	  case 0xc4: p[i]=0x8e;break;
-	  case 0xd6: p[i]=0x99;break;
-	  case 0xdc: p[i]=0x9a;break;
-	  case 0xe4: p[i]=0x84;break;
-	  case 0xf6: p[i]=0x94;break;
-	  case 0xfc: p[i]=0x81;break;
-	  case 0xdf: p[i]=0xe1;break;
-	  }
-      
-      return 1;
+
+  strcpy(orig,pw);
+  combinations=1;
+
+  for (i=0; i<l; i++)
+    combinations *=is_multi[pw[i]];
+
+if (!insist) {
+    max = max_multi;
+    while (combinations > RESOLV_MAX_COMBINATIONS) {
+      max--;
+      combinations = 1;
+      for (i=0; i<l; i++)
+	combinations *=MIN(is_multi[pw[i]],max);
     }
   }
-  return 0;
+
+  update_statusbar(MSG_RESOLV_NT_HASH,(int)combinations);
+  while (gtk_events_pending())
+    gtk_main_iteration();
+
+
+  for (i=0; i<combinations; i++) {
+    int c,m;
+    c=i;
+
+    for (j=0; j<l; j++) {
+      m = MIN(is_multi[orig[j]],max);
+      pw[j]=((m>1) ? multi_char[orig[j]][c%m]:orig[j]);
+      c/=m;
+    }
+    md4[16]=0;
+    make_nthash(pw,(char *)md4);
+ 
+    if (!memcmp(md4,h,16)) 
+      return 1;
+
+    if (!(i %17123)){
+      update_statusbar(MSG_RESOLV_NT_HASH,(int)(combinations-i));
+      while (gtk_events_pending())
+	gtk_main_iteration();
+      if (STOP) return 0;
+    }
+    
+  }
+
+  bzero(pw,16);
+  return 0;    
 }
 
 
@@ -1038,7 +1113,7 @@ void display(int h, int k)
       for (i=0; i<8; i++) printf("%02hhx",lm_hash2[h][i]);
       printf(" : ");
     }
-    if (resolve_nt_hash(password1[h],password2[h], (char *)nt_hash[h],password[h]) )
+    if (resolve_nt_hash(password1[h],password2[h], (char *)nt_hash[h],password[h], RESOLV_DO_NOT_INSIST))
       printf("%-14s    \n",password[h]);
     else
     printf("%-7s%-7s    \n",password1[h],password2[h]);
@@ -1355,12 +1430,13 @@ int evaluate_ram() {
 }
 
 #ifdef LIVECD
-int init_livecd(char *dir) {
-
+int init_livecd(char* dir[]) {
   bzero(directory, strlen(directory));
-  strncpy(directory, dir, strlen(dir));
+  strncpy(directory, dir[1], strlen(dir[1]));
   auto_detect_tables(directory);
 
+  extract_hashes_from_dir(dir[2]);
+
   activate_launch_toggle();
 
 }
@@ -1547,7 +1623,7 @@ int main_cmd()
 	 if (!(k%display_step)) 
 	  display (-2,k);
       */
-      update_statusbar(k);
+      update_statusbar(MSG_DEFAULT,k);
 
       /* for each file containing a table: */
       for (table=first_table; table<=last_table; table++) {
@@ -1572,11 +1648,11 @@ int main_cmd()
 		  if (done_nt[h])
 		    n_found++;
 		  else 
-		    if (resolve_nt_hash(password1[h],password2[h],(char *)nt_hash[h],password[h]))
+		    if (resolve_nt_hash(password1[h],password2[h],(char *)nt_hash[h],password[h], RESOLV_DO_NOT_INSIST))
 		      {n_found++; to_go--; done_nt[h]=1;}
 		  change_tree(h); 
 		  //display(h,k);
-		  update_statusbar(k);
+		  update_statusbar(MSG_DEFAULT,k);
 		}
 		if (!to_go) break; 
 	      }
@@ -1592,11 +1668,11 @@ int main_cmd()
 		  if (done_nt[h])
 		    n_found++;
 		  else 
-		    if (resolve_nt_hash(password1[h],password2[h],(char *)nt_hash[h],password[h]))
+		    if (resolve_nt_hash(password1[h],password2[h],(char *)nt_hash[h],password[h], RESOLV_DO_NOT_INSIST))
 		      {n_found++; to_go--; done_nt[h]=1;}
 		  change_tree(h);
 		  //display(h,k);
-		  update_statusbar(k);}
+		  update_statusbar(MSG_DEFAULT,k);}
 		if (!to_go) break; 
 	      }
 	    }
@@ -1614,7 +1690,7 @@ int main_cmd()
 		if (!done2[h])
 		  {to_go--; done2[h]=1;}
 		
-		update_statusbar(k);
+		update_statusbar(MSG_DEFAULT,k);
 		if (!to_go) break; 
 	      }
 	    }
